/*
 * @package    Revo Debug System Plugin
 * @version    0.1.0
 * @author     Alex Revo - alexrevo.pw
 * @copyright  Copyright (c) 2021 Alex Revo. All rights reserved.
 * @license    GPL
 * @link       https://alexrevo.pw/
 */
let issuecount = 0;

if (!window.jQuery) {
  var tag = document.createElement("script");
  tag.src = "https://ajax.googleapis.com/ajax/libs/jquery/3.6.0/jquery.min.js";
  document.querySelector("head").appendChild(tag);
}

function sendnotify(icon, clipboard, status, timeout) {
  UIkit.notification({
    message: "<span uk-icon='icon: " + icon + "'></span> " + clipboard,
    pos: "bottom-left",
    status: status,
    timeout: timeout,
  });
}

function log(p1, p2, p3, color) {
  switch (color) {
    case "green":
      color = "#539e09";
      break;
    case "red":
      color = "#ff3e4b";
      issuecount = issuecount + 1;
      break;
    case "blue":
      color = "#018a9c";
      break;
    case "orange":
      color = "orange";
      break;
    default:
      color = "#018a9c";
  }
  console.log("%c" + p1 + p2 + p3, "color:" + color);
}

function byteCount(s) {
  return encodeURI(s).split(/%..|./).length - 1;
}

function isValidJson(json) {
  try {
    JSON.parse(json);
    return true;
  } catch (e) {
    return false;
  }
}

setTimeout(function () {
  if (!String.prototype.startsWith) {
    Object.defineProperty(String.prototype, "startsWith", {
      enumerable: false,
      configurable: false,
      writable: false,
      value: function (searchString, position) {
        position = position || 0;
        return this.indexOf(searchString, position) === position;
      },
    });
  }

  jQuery(document).ready(function ($) {
    function runDebugger() {
      let Yoopanel = jQuery(".yo-sidebar-footer").length > 0;
      if (!Yoopanel) {
        setTimeout(runDebugger, 2000);
        return;
      }

      /*
       * Add toogle to the Footer
       */

      let html =
        '<li data-index="6" style="" class="uk-animation-slide-bottom-small"><a uk-icon="lifesaver" uk-tooltip="Debug report" class="uk-icon uk-icon-link debugger"></a></li>';
      jQuery(".yo-sidebar-footer ul.uk-grid").append(html);
      jQuery(".debugger").css("opacity", "0.35");
    }

    runDebugger();
  });
}, 3500);

setTimeout(function ($) {
  jQuery(document).ready(function () {
    jQuery(document).on("click", ".debugger", function (e) {
      e.preventDefault();
      issuecount = 0;
      let layout = document.querySelector(
        "body > div.yo-customizer > div.yo-preview > div > iframe"
      ).contentWindow.$customizer;

      jQuery(this).css("opacity", "1");
      sendnotify(
        "lifesaver",
        "open the browser's console log",
        "success",
        3000
      );

      valid = isValidJson(JSON.stringify($customizer));
      if (valid == true) {
        log("Customizer loaded", "", "", "green");
      } else {
        log("Customizer error", "", "", "red");
      }

      if (typeof layout.page !== "undefined") {
        log("↓↓↓ - Current JSON layout - ↓↓↓", "", "", "blue");
        console.log(JSON.stringify(layout.page.content));
        if (document.querySelector("textarea.markup").length) {
          document.querySelector("textarea.markup").value = JSON.stringify(
            layout.page.content
          );
        }
        valid = isValidJson(JSON.stringify(layout.page.content));
        if (valid == true) {
          log("This is valid JSON", "", "", "green");
        } else {
          log("This is invalid JSON", "", "", "red");
        }
        log(
          "→ → → Check the markup also on https://jsonlint.com",
          "",
          "",
          "blue"
        );

        size = ((byteCount(layout.page.content) * 100) / 16777215).toFixed(2);
        if (size < 70) {
          log("Current layout size: ", size, "%", "green");
        } else if (size < 90) {
          log("Current layout size: ", size, "%", "orange");
        } else if (size > 90) {
          log("Current layout size: ", size, "%", "red");
        }
      } else if (typeof layout.template !== "undefined") {
        log("Current template:", "", layout.template, "green");
        log("↓↓↓ - JSON layout - ↓↓↓", "", "", "blue");
        console.log(JSON.stringify($builder.templates));
        valid = isValidJson(JSON.stringify($builder.templates));
        if (valid == true) {
          log("This is valid JSON", "", "", "green");
        } else {
          log("This is invalid JSON", "", "", "red");
        }
        console.log(
          "→ → → Check the markup also on https://jsonlint.com",
          "",
          "",
          "blue"
        );
      }
      if (PBT) {
        //max input vars
        if (PBT.MIV >= 10000) {
          log("Max Input Vars: ", PBT.MIV, " | recommended: 10 000", "green");
        } else {
          log("Max Input Vars: ", PBT.MIV, " | recommended: 10 000", "red");
        }
        //post max size
        if (parseInt(PBT.PMS) >= 16) {
          log("Post Max Size: ", PBT.PMS, " | recommended: > 16M", "green");
        } else {
          console.log(
            "Post Max Size: ",
            PBT.PMS,
            " | recommended: > 16M",
            "red"
          );
        }
        //memory limit
        if (parseInt(PBT.MEM) >= 512) {
          log("Memory limit: ", PBT.MEM, " | recommended (min): 512M", "green");
        } else {
          log("Memory limit: ", PBT.MEM, " | recommended (min): 512M", "red");
        }
        //progressive cache
        if (PBT.SCH == 2) {
          log(
            "Joomla Progressive Cache Mode is active ",
            "| ",
            "recommended use Conservative Mode",
            "red"
          );
        }
        //page cache plugin
        if (PBT.PCP == 1) {
          log(
            "Joomla Page Cache System plugin is active ",
            "| ",
            "recommended disable Page Cache plugin",
            "red"
          );
        }
        //check chared session
        if (PBT.SES == 1) {
          log(
            "Joomla Shared session mode is active",
            " | ",
            "recommended turn off shared session",
            "red"
          );
        }
      }
      if (issuecount) {
        sendnotify(
          "lifesaver",
          "Found " + issuecount + " issue(s)",
          "danger",
          3000
        );
      }
      log("Total issue(s): ", issuecount, "", "blue");
      log(
        "Go to the theme support site: ",
        "https://yootheme.com/support",
        "",
        "green"
      );
      log("Hire an expert: ", "https://alexrevo.pw/services/", "", "orange");
      log(
        "Report created by Revo Debug System plugin ",
        "v.0.1.0-beta",
        "",
        "blue"
      );
      jQuery(this).css("opacity", "0.35");
    });
  });
}, 1000);
